<?PHP
/*
 * DIREITOS AUTORAIS RESERVADOS (C) 2007 UFRJ
 * 
 * ESTE PROGRAMA É SOFTWARE LIVRE; VOCÊ PODE REDISTRIBUÍ-LO E/OU MODIFICÁ-LO SOB OS
 * TERMOS DA LICENÇA PÚBLICA GERAL GNU CONFORME PUBLICADA PELA FREE SOFTWARE FOUNDATION;
 * TANTO A VERSÃO 2 DA LICENÇA, COMO (A SEU CRITÉRIO) QUALQUER VERSÃO POSTERIOR.

 * ESTE PROGRAMA É DISTRIBUÍDO NA EXPECTATIVA DE QUE SEJA ÚTIL, PORÉM, SEM NENHUMA GARANTIA;
 * NEM MESMO A GARANTIA IMPLÍCITA DE COMERCIABILIDADE OU ADEQUAÇÃO A UMA FINALIDADE ESPECÍFICA.
 * CONSULTE A LICENÇA PÚBLICA GERAL DO GNU PARA MAIS DETALHES.
 * 
 * VOCÊ DEVE TER RECEBIDO UMA CÓPIA DA LICENÇA PÚBLICA GERAL DO GNU JUNTO COM ESTE PROGRAMA; 
 * SE NÃO, ACESSE <<HTTP://WWW.GNU.ORG/LICENSES/LICENSES.PT-BR.HTML>>
 */

<?php
/*********************************************************************

Programa: AS10.PHP
Criado em: 27/02/2010
Autor: Gustavo Moreira

Definição:
    Esse script calcula o indicador AS10 - Grau de cobertura protetiva eventual

    
Cálculo:
    (Número de famílias que declararam receber apenas benefícios eventuais) / (Número de famílias atendidas nos CRAS) x 100
    
Periodicidade:
     Mensal, trimestral e Anual
     
Granularidade:
    Município (todos os CRAS)
    
OBS:

São considerados benefícios eventuais: cestas básicas, auxílio-funeral, aluguel social, Programa do Leite. 
Entram no numerador apenas aquelas que famílias que só recebem esse tipo de benefício, referido a partir da Ficha de Cadastramento do Usuário do CRAS.

    

Parâmetros de Entrada (via GET):

    ...?indicador=III&granularidade=GGG&periodo=PPP
    III = no indicador, em caixa baixa, é informado o seu código
    GGG = na granularidade, em caixa baixa e sem acentuação, é informada o nome da granularidade desejada
    PPP = no periodo, em se tratando de um indicador mensal, é informado o mês e ano, no formato MM/AAAA (exs: 01/2010, 02/2010 etc)
    
    Exemplo de Entrada:
    ...?indicador=as2&granularidade=cras&periodo=01/2010
    
Saída:
    Formato do retorno
        CCC;III;G1;PPP;VVV
        CCC = código de retorno da chamada ao Calculador
        III = código do indicador retornado, em caixa baixa
        G1 = nome do grão retornado (string, pode estar acentuado e com letras maíusculas e minúsculas)
        PPP = período de referência do valor retornado
        VVV = valor retornado
        
    Exemplos de retorno:
    
        0;as2;CRAS;1;01/2010;13,2
        0;as2;CRAS;10;01/2010;10,2
        0;as2;CRAS Austin;11;01/2010;11
        

        Códigos de Retorno:

0 - Sucesso
1 - Não tenho valor
2 - Não sou eu
3 - Um ou mais Parâmetros obrigatórios faltando
4 - Um ou mais Parâmetros fora do formato esperado
5 - Granularidade invalida

*********************************************************************/


/* PARÂMETROS DE ENTRADA */
$indicador = $_GET["indicador"];
$granularidade = $_GET["granularidade"];
$periodo = $_GET["periodo"];

/* VALIDAR A ENTRADA DE DADOS */

//obrigatoriedade dos parâmetros
if (($indicador == "") || ($granularidade == "") || ($periodo == ""))
{
    echo "3";
    exit;
}

//indicador correto
if (strtoupper($indicador) != "AS10")
{
    echo "2";
    exit;
}

if (($granularidade != "município") && ($granularidade != "municipio"))
{
    echo "5";
    exit;
}

//verificar se é o período válido

//verifica se é anual
if (!strpos($periodo,"/"))
{   
    $tipo_periodo = "anual";
    if (!is_numeric($periodo))
    {
        echo "4";
        exit;
    }
    $data_ini = "$periodo-01-01 00:00:00";
    $data_fim = "$periodo-12-31 23:59:59";
}
else
{   
    //é mensal
    if ($periodo[0] != "T")
    {
        
        list($mes,$ano) = split("/",$periodo);
        $data_aux = date_create("$mes/01/$ano"); //formato do date_create (mes/dia/ano)
                
        if (!$data_aux)
        {
            echo "4";
            exit;    
        };
        
        $data_ini = "$ano-$mes-01 00:00:00";
        //traz o ultimo dia do mes através do formato "t"
        $data_fim = date("Y-m-t H:i:s", mktime(23, 59, 59, $mes, 1, $ano));
        
    }
    else
    {   //trimestral
        list($trimestre,$ano) = split("/",$periodo);
        
        if (($trimestre!="T1") && ($trimestre!="T2") && ($trimestre!="T3") && ($trimestre!="T4"))
        {
             echo "4";
             exit;
        }
        
        if ($trimestre=="T1")
        {
            $data_ini = "$ano-01-01 00:00:00";
            $data_fim = "$ano-03-31 23:59:59";    
        }
        
        if ($trimestre=="T2")
        {
            $data_ini = "$ano-04-01 00:00:00";
            $data_fim = "$ano-06-30 23:59:59";    
        }
        
        if ($trimestre=="T3")
        {
            $data_ini = "$ano-07-01 00:00:00";
            $data_fim = "$ano-09-30 23:59:59";
        }
        
        if ($trimestre=="T4")
        {
            $data_ini = "$ano-10-01 00:00:00";
            $data_fim = "$ano-12-31 23:59:59";
        }
        
    }
}


/* INICIAR CONEXÃO COM BANCO DE DADOS (CRAS) */
include "../includes/begin.php";


/* Calculo Total de Visitas no periodo */
$sql = "SELECT  COUNT(*) as valor 
        FROM VISITA 
        WHERE DATAVISITA  >= '$data_ini' and DATAVISITA  <= '$data_fim' ";

$result=pg_query($dbh, $sql);    

if (pg_num_rows($result) == 0 )
{
    echo "1";
    exit;
}

$row = pg_fetch_array($result);
$tot_atendimentos = $row["valor"];

//evitar divisão por zero
if ($tot_atendimentos == 0)
{
    echo "1";
    exit;
}


/* Calculo Total de benef eventuais no periodo */
$sql = "SELECT  numerocras, datacadastrousuario, COUNT(*) as valor 
        FROM VISITA v
        WHERE DATAVISITA  >= '$data_ini' and DATAVISITA  <= '$data_fim' 
        AND exists (select 1 from usuario u 
                    where u.numerocras = v.numerocras and u.datacadastrousuario = v.datacadastrousuario 
                    and     exists ( select 1 from usuariobeneficio bf where u.numerocras = bf.numerocras and u.datacadastrousuario = bf.datacadastrousuario and  bf.respostabeneficio = 'S' and bf.beneficio in (8,14) )
                    and not exists ( select 1 from usuariobeneficio bf where u.numerocras = bf.numerocras and u.datacadastrousuario = bf.datacadastrousuario and  bf.respostabeneficio = 'S' and bf.beneficio not in (8,14) )                        
                    ) 
        group by numerocras, datacadastrousuario";
        
        
$result=pg_query($dbh, $sql);    

if (pg_num_rows($result) == 0 )
{
    echo "1";
    exit;
}

$row = pg_fetch_array($result);
$tot_eventuais = $row["valor"];
        

$valor = round(($tot_eventuais/$tot_atendimentos)*100,4);
$valor = str_replace(".",",",$valor);
        
//Escrever a resposta
echo "0;$indicador;Nova Iguaçu;$periodo;$valor";


/* ENCERRAR CONEXÃO COM BANCO DE DADOS */
include "../includes/end.php";
?>